<?php

namespace common\logic\Product\Product;

use common\logic\BaseService;
use common\logic\PointSale\PointSale\PointSale;
use common\logic\Product\ProductPrice\ProductPriceSolver;
use common\logic\SolverInterface;
use common\logic\User\User\User;

class ProductSolver extends BaseService implements SolverInterface
{
    protected ProductPriceSolver $productPriceSolver;

    public function __construct()
    {
        $this->productPriceSolver = $this->loadService(ProductPriceSolver::class);
    }

    public function getPrice(Product $product, array $params = []): float
    {
        $specificPriceArray = $product->productPrice;

        $user = isset($params['user']) ? $params['user'] : false;
        $userProducer = isset($params['user_producer']) ? $params['user_producer'] : false;
        $pointSale = isset($params['point_sale']) ? $params['point_sale'] : false;
        $quantity = (isset($params['quantity']) && $params['quantity']) ? $params['quantity'] : 1;

        if ($specificPriceArray && ($user || $pointSale)) {
            $specificPriceArray = $this->productPriceSolver->filterByPriorityMatch(
                $specificPriceArray,
                $user,
                $pointSale
            );
            $bestPrice = 9999;
            foreach ($specificPriceArray as $specificPrice) {
                $fromQuantity = $specificPrice->from_quantity;
                if ((($fromQuantity && $fromQuantity <= $quantity) || !$fromQuantity)
                    && $specificPrice->price < $bestPrice) {
                    $bestPrice = $specificPrice->price;
                }
            }

            if ($bestPrice != 9999) {
                return $bestPrice;
            }
        }

        if ($userProducer && $userProducer->product_price_percent) {
            return $product->price * (1 + $userProducer->product_price_percent / 100);
        }

        if ($pointSale && $pointSale->product_price_percent) {
            return $product->price * (1 + $pointSale->product_price_percent / 100);
        }

        return $product->price;
    }
}